pub struct MarkedArena<'a, R: for<'b> Rootable<'b>>(/* private fields */);Implementations
Sourceimpl<R> MarkedArena<'_, R>
impl<R> MarkedArena<'_, R>
Sourcepub fn finalize<F, T>(self, f: F) -> T
pub fn finalize<F, T>(self, f: F) -> T
Examine the state of a fully marked arena.
Allows you to determine whether GcWeak pointers are “dead” (aka, soon-to-be-dropped) and
potentially resurrect them for this cycle.
Note that the arena is guaranteed to be fully marked only at the beginning of this callback, any mutation that resurrects a pointer or triggers a write barrier can immediately invalidate this.
Sourcepub fn start_sweeping(self)
pub fn start_sweeping(self)
Immediately transition the arena out of CollectionPhase::Marked to
CollectionPhase::Sweeping.