pub trait Rootable<'a> {
type Root: ?Sized + 'a;
}Expand description
A trait that produces a Collect-able type for the given lifetime. This is used to produce
the root Collect instance in an Arena.
In order to use an implementation of this trait in an Arena, it must implement
Rootable<'a> for any possible 'a. This is necessary so that the Root types can be
branded by the unique, invariant lifetimes that makes an Arena sound.