Write

Struct Write 

#[non_exhaustive]
pub struct Write<T: ?Sized> { /* private fields */ }
An (interiorly-)mutable reference inside a GC’d object graph.

This type can only exist behind a reference; it is typically obtained by calling Gc::write on a Gc pointer or by using the field! projection macro on a pre-existing &Write<T>.

Implementations

Source

impl<T: ?Sized> Write<T>

pub unsafe fn assume(v: &T) -> &Self

Asserts that the given reference can be safely written to.

Safety

In order to maintain the invariants of the garbage collector, no new Gc pointers may be adopted by the referenced value as a result of the interior mutability enabled by this wrapper, unless Gc::write is invoked manually on the parent Gc pointer during the current arena callback.

pub fn from_static(v: &T) -> &Self
where T: 'static,

Gets a writable reference to non-GC’d data.

This is safe, as 'static types can never hold Gc pointers.

pub fn from_mut(v: &mut T) -> &mut Self

Gets a writable reference from a &mut T.

This is safe, as exclusive access already implies writability.

pub fn unlock(&self) -> &T::Unlocked
where T: Unlock,

Unlocks the referenced value, providing full interior mutability.

Source

impl<T> Write<Option<T>>

pub fn as_write(&self) -> Option<&Write<T>>

Converts from &Write<Option<T>> to Option<&Write<T>>.

Source

impl<T, E> Write<Result<T, E>>

pub fn as_write(&self) -> Result<&Write<T>, &Write<E>>

Converts from &Write<Result<T, E>> to Result<&Write<T>, &Write<E>>.