pub struct Lock<T: ?Sized> { /* private fields */ }Expand description
Implementations
Sourceimpl<T: ?Sized> Lock<T>
impl<T: ?Sized> Lock<T>
pub fn as_ptr(&self) -> *mut T
Sourcepub unsafe fn as_cell(&self) -> &Cell<T>
pub unsafe fn as_cell(&self) -> &Cell<T>
Access the wrapped Cell.
Safety
In order to maintain the invariants of the garbage collector, no new Gc
pointers may be adopted by this type as a result of the interior mutability
afforded by directly accessing the inner Cell, unless the write barrier for the containing Gc pointer is invoked manually
before collection is triggered.
pub fn get_mut(&mut self) -> &mut T