#[repr(transparent)]pub struct Static<T: ?Sized>(pub T);Expand description
A wrapper type that implements Collect whenever the contained T is ’static, which is useful in generic contexts
Tuple Fields
0: TTrait Implementations
Sourceimpl<T: ?Sized> BorrowMut<T> for Static<T>
impl<T: ?Sized> BorrowMut<T> for Static<T>
Sourcefn borrow_mut(&mut self) -> &mut T
fn borrow_mut(&mut self) -> &mut T
Mutably borrows from an owned value. Read more
Sourceimpl<T: ?Sized + 'static> Collect for Static<T>
impl<T: ?Sized + 'static> Collect for Static<T>
Sourcefn needs_trace() -> bool
fn needs_trace() -> bool
As an optimization, if this type can never hold a
Gc pointer and trace is unnecessary
to call, you may implement this method and return false. The default implementation returns
true, signaling that Collect::trace must be called.Sourcefn trace(&self, _cc: &Collection)
fn trace(&self, _cc: &Collection)
Must call
Collect::trace on all held Gc pointers. If this type holds inner types that
implement Collect, a valid implementation would simply call Collect::trace on all the
held values to ensure this.Sourceimpl<T: Ord + ?Sized> Ord for Static<T>
impl<T: Ord + ?Sized> Ord for Static<T>
Sourceimpl<T: PartialOrd + ?Sized> PartialOrd for Static<T>
impl<T: PartialOrd + ?Sized> PartialOrd for Static<T>
impl<T: Copy + ?Sized> Copy for Static<T>
impl<T: Eq + ?Sized> Eq for Static<T>
impl<T: ?Sized> StructuralPartialEq for Static<T>