pub enum CollectionPhase {
Sleeping,
Marking,
Marked,
Sweeping,
}Variants
Sleeping
The arena is done with a collection cycle and is waiting to be restarted.
Marking
The arena is currently tracing objects from the root to determine reachability.
Marked
The arena has finished tracing, all reachable objects are marked. This may transition
back to Marking if write barriers occur.
Sweeping
The arena has determined a set of unreachable objects and has started freeing them. At this point, marking is no longer taking place so the root may have reachable, unmarked pointers.